# Cohoba - a GNOME client for Telepathy
# Derived from telepathy-python
#
# Copyright (C) 2005 Collabora Limited
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import re
import dbus, dbus.glib
import dbus.service

from telepathy.interfaces import CONN_INTERFACE

from cohoba.common.TelepathyConstants import (MISSION_CONTROL_INTERFACE,
	MISSION_CONTROL_BUS_NAME, MISSION_CONTROL_OBJECT_PATH,
	MISSION_CONTROL_UI_INTERFACE, CONNECTION_PRESENCE_OFFLINE)
from cohoba.common.Utils import singleton, global_error_handler
from cohoba.common.DBusProxyProvider import get_proxy_provider
from cohoba.common.Store import Store

from cohoba.houston.AccountList import AccountList
from cohoba.houston.ChannelHandler import ChannelHandler

CONNECTION_MATCH = re.compile(r'^org\.freedesktop\.Telepathy\.Connection\.([^.]+)\.([^.]+)(\..*)$')
def telepathy_service_to_obj_path(service):
	return "/%s" % re.sub(r"\.", "/", service)

class PresenceStore(Store):
	def __init__(self):
		Store.__init__(self, "houston-presence")
PresenceStore = singleton(PresenceStore)

class Houston(dbus.service.Object):	
	def __init__(self):
		bus_name = MISSION_CONTROL_BUS_NAME
		object_path = MISSION_CONTROL_OBJECT_PATH
		dbus.service.Object.__init__(self, dbus.service.BusName(bus_name), object_path)
		print 'Houston listening'
		
		self.presence = PresenceStore().load((CONNECTION_PRESENCE_OFFLINE, ""))
		
		self.accounts = AccountList()
		# Channel handler listens for account additions, put it before loading
		self.channel_handler = ChannelHandler(self.accounts)

		self.accounts.load()
		self.hook_active_connections()
		
		self.accounts.connect('account-added', self.on_account_added)
		for account in self.accounts:
			self.on_account_added(self.accounts, account)
			self.on_account_changed(account)
		
	def hook_active_connections(self):
		get_proxy_provider().create_proxy(
				"org.freedesktop.DBus",
				"/org/freedesktop/DBus")

		dbus_iface = get_proxy_provider().get_iface(
			"/org/freedesktop/DBus",
			"org.freedesktop.DBus")
		
		try:
			print 'Hooking-up to existing connections on the bus'
			self.got_names(dbus_iface.ListNames())
		except dbus.DBusException, e:
			global_error_handler(e)
	
	def got_names(self, names):
		for name in names:
			match = CONNECTION_MATCH.match(name)
			if match != None:
				conn_name, conn_path = name, telepathy_service_to_obj_path(name)
				manager, protocol, connmgrinfo = match.group(1), match.group(2), match.group(3)
				account = self.accounts.find(conn_path)
				if account != None:
					print 'Found active connection when starting, hookingup'
					get_proxy_provider().create_proxy(conn_name, conn_path)
					account.connect_account_to_connection(conn_name, conn_path)
	
	def on_account_added(self, accountlist, account):
		account.connect('account-changed', self.on_account_changed)
	
	def on_account_changed(self, account):
		self.set_presence(account)
		if account.is_connected():
			self.AccountConnected(get_proxy_provider().get_service(account.conn_obj), account.conn_obj)
			
	def set_presence(self, account):
		# First try to disconnect if we are disabled
		if not account.enabled:
			print 'Account disabled, disconnecting if necessary:', account
			account.disconnect_account()
			return
					
		# We want to be online, but the account is disconnected, connect it
		if self.presence[0] != CONNECTION_PRESENCE_OFFLINE and \
					not account.is_connected() and \
					account.is_connectable():
			print 'Presence is != OFFLINE, and the account is connectable but not connected. Connecting:', account
			account.connect_account()
		# We want to be offline, bu the account is connected, disconnect it
		elif self.presence[0] == CONNECTION_PRESENCE_OFFLINE and account.is_connected():
			print 'Presence is == OFFLINE, disconnect account if necessary:', account
			account.disconnect_account()
		
		# Now we want to be online with a specified presence setting, let's do it
		if self.presence[0] != CONNECTION_PRESENCE_OFFLINE and \
				account.is_connected():
			account.set_presence(*self.presence)
	
	def exit(self):
		print 'Houston is exiting. Disconnecting every connection.'
		for account in self.accounts:
			if account.is_connected():
				try:
					account.disconnect_account()
				except Exception, e:
					print '--Unable to disconnect:', e, ':', account
		
	# --- Methods ---
	@dbus.service.method(MISSION_CONTROL_INTERFACE, in_signature='ss', out_signature='')
	def SetPresence(self, name, message):
		self.presence = (name, message)
		PresenceStore().save(self.presence)
		
		for account in self.accounts:
			self.set_presence(account)
	
	@dbus.service.method(MISSION_CONTROL_INTERFACE, in_signature='', out_signature='ss')
	def GetPresence(self):
		return self.presence
			
	@dbus.service.method(MISSION_CONTROL_INTERFACE, in_signature='', out_signature='a(soss)')
	def GetConnectedAccounts(self):
		result = []
		for account in self.accounts:
			if not account.is_connected():
				continue
				
			result += [(get_proxy_provider().get_service(account.conn_obj), account.conn_obj, account.profile.protocol, account.get_username())]
		
		return result
	
	@dbus.service.method(MISSION_CONTROL_INTERFACE, in_signature='sosuu', out_signature='')
	def RequestChannel(self, conn_name, conn_obj, chan_type, handle_type, handle):
		conn_iface = get_proxy_provider().get_iface(conn_obj, CONN_INTERFACE)
		conn_iface.RequestChannel(chan_type, handle_type, handle, True,
			reply_handler=lambda chan_obj: self.channel_handler.handle_channel(conn_name, conn_obj, chan_obj, chan_type, handle_type, handle),
			error_handler=global_error_handler)
	
	@dbus.service.method(MISSION_CONTROL_UI_INTERFACE, in_signature='', out_signature='')
	def ShowAccounts(self):
		from cohoba.houston.ui.AccountsWindow import AccountsWindow
		AccountsWindow().show()
	
	# --- Signals ---
	@dbus.service.signal(MISSION_CONTROL_INTERFACE, signature='so')
	def AccountConnected(self, bus_name, object_path):
		self.channel_handler.handle_connection(bus_name, object_path)

Houston = singleton(Houston)
